@@ -18,7 +18,7 @@
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  */
-#include <config.h>
+#include "config.h"
 #include <errno.h>
 #include <math.h>
 #include <stdarg.h>
@@ -32,17 +32,11 @@
 
 #include "viewer-render.h"
 
-#define DEFAULT_FONT_FAMILY "Sans"
-#define DEFAULT_FONT_SIZE 18
-
-#define _MAKE_FONT_NAME(family, size) family " " #size
-#define MAKE_FONT_NAME(family, size) _MAKE_FONT_NAME(family, size)
-
 const char *prog_name;
 
 gboolean opt_display = TRUE;
 int opt_dpi = 96;
-const char *opt_font = MAKE_FONT_NAME (DEFAULT_FONT_FAMILY, DEFAULT_FONT_SIZE);
+const char *opt_font = "";
 gboolean opt_header = FALSE;
 const char *opt_output = NULL;
 int opt_margin = 10;
@@ -53,8 +47,11 @@ gboolean opt_auto_dir = TRUE;
 const char *opt_text = NULL;
 gboolean opt_waterfall = FALSE;
 int opt_width = -1;
+int opt_height = -1;
 int opt_indent = 0;
+gboolean opt_justify = 0;
 int opt_runs = 1;
+PangoAlignment opt_align = PANGO_ALIGN_LEFT;
 PangoEllipsizeMode opt_ellipsize = PANGO_ELLIPSIZE_NONE;
 PangoGravity opt_gravity = PANGO_GRAVITY_SOUTH;
 PangoGravityHint opt_gravity_hint = PANGO_GRAVITY_HINT_NATURAL;
@@ -63,6 +60,8 @@ PangoWrapMode opt_wrap = PANGO_WRAP_WORD_CHAR;
 gboolean opt_wrap_set = FALSE;
 const char *opt_pangorc = NULL;
 const PangoViewer *opt_viewer = NULL;
+const char *opt_language = NULL;
+gboolean opt_single_par = FALSE;
 
 /* Text (or markup) to render */
 static char *text;
@@ -80,27 +79,13 @@ fail (const char *format, ...)
   exit (1);
 }
 
-static PangoFontDescription *
-get_font_description (void)
-{
-  PangoFontDescription *font_description = pango_font_description_from_string (opt_font);
-
-  if ((pango_font_description_get_set_fields (font_description) & PANGO_FONT_MASK_FAMILY) == 0)
-    pango_font_description_set_family (font_description, DEFAULT_FONT_FAMILY);
-
-  if ((pango_font_description_get_set_fields (font_description) & PANGO_FONT_MASK_SIZE) == 0)
-    pango_font_description_set_size (font_description, DEFAULT_FONT_SIZE * PANGO_SCALE);
-
-  return font_description;
-}
-
 static PangoLayout *
 make_layout(PangoContext *context,
 	    const char   *text,
 	    double        size)
 {
   static PangoFontDescription *font_description;
-  PangoDirection base_dir;
+  PangoAlignment align;
   PangoLayout *layout;
 
   layout = pango_layout_new (context);
@@ -111,23 +96,35 @@ make_layout(PangoContext *context,
 
   pango_layout_set_auto_dir (layout, opt_auto_dir);
   pango_layout_set_ellipsize (layout, opt_ellipsize);
+  pango_layout_set_justify (layout, opt_justify);
+  pango_layout_set_single_paragraph_mode (layout, opt_single_par);
+  pango_layout_set_wrap (layout, opt_wrap);
 
-  font_description = get_font_description ();
+  font_description = pango_font_description_from_string (opt_font);
   if (size > 0)
     pango_font_description_set_size (font_description, size * PANGO_SCALE);
 
   if (opt_width > 0)
-    {
-      pango_layout_set_wrap (layout, opt_wrap);
-      pango_layout_set_width (layout, (opt_width * opt_dpi * PANGO_SCALE + 36) / 72);
-    }
+    pango_layout_set_width (layout, (opt_width * opt_dpi * PANGO_SCALE + 36) / 72);
+
+  if (opt_height > 0)
+    pango_layout_set_height (layout, (opt_height * opt_dpi * PANGO_SCALE + 36) / 72);
+  else
+    pango_layout_set_height (layout, opt_height);
 
   if (opt_indent != 0)
     pango_layout_set_indent (layout, (opt_indent * opt_dpi * PANGO_SCALE + 36) / 72);
 
-  base_dir = pango_context_get_base_dir (context);
-  pango_layout_set_alignment (layout,
-			      base_dir == PANGO_DIRECTION_LTR ? PANGO_ALIGN_LEFT : PANGO_ALIGN_RIGHT);
+  align = opt_align;
+  if (align != PANGO_ALIGN_CENTER &&
+      pango_context_get_base_dir (context) != PANGO_DIRECTION_LTR) {
+    /* pango reverses left and right if base dir ir rtl.  so we should
+     * reverse to cancel that.  unfortunately it also does that for
+     * rtl paragraphs, so we cannot really get left/right.  all we get
+     * is default/other-side. */
+    align = PANGO_ALIGN_LEFT + PANGO_ALIGN_RIGHT - align;
+  }
+  pango_layout_set_alignment (layout, align);
 
   pango_layout_set_font_description (layout, font_description);
 
@@ -139,7 +136,7 @@ make_layout(PangoContext *context,
 gchar *
 get_options_string (void)
 {
-  PangoFontDescription *font_description = get_font_description ();
+  PangoFontDescription *font_description = pango_font_description_from_string (opt_font);
   gchar *font_name;
   gchar *result;
 
@@ -155,8 +152,7 @@ get_options_string (void)
 }
 
 static void
-output_body (PangoContext   *context,
-	     const char     *text,
+output_body (PangoLayout    *layout,
 	     RenderCallback  render_cb,
 	     gpointer        cb_context,
 	     gpointer        cb_data,
@@ -164,7 +160,6 @@ output_body (PangoContext   *context,
 	     int            *height,
 	     gboolean        supports_matrix)
 {
-  PangoLayout *layout;
   PangoRectangle logical_rect;
   int size, start_size, end_size, increment;
   int x = 0, y = 0;
@@ -173,6 +168,7 @@ output_body (PangoContext   *context,
     {
       const PangoMatrix* matrix;
       const PangoMatrix identity = PANGO_MATRIX_INIT;
+      PangoContext *context = pango_layout_get_context (layout);
       matrix = pango_context_get_matrix (context);
       if (matrix)
 	{
@@ -180,6 +176,7 @@ output_body (PangoContext   *context,
 	  y += matrix->y0;
 	}
       pango_context_set_matrix (context, &identity);
+      pango_layout_context_changed (layout);
     }
 
   if (opt_waterfall)
@@ -199,17 +196,24 @@ output_body (PangoContext   *context,
 
   for (size = start_size; size <= end_size; size += increment)
     {
-      layout = make_layout (context, text, size);
-      pango_layout_get_extents (layout, NULL, &logical_rect);
+      if (size > 0)
+        {
+	  PangoFontDescription *desc = pango_font_description_copy (pango_layout_get_font_description (layout));
+	  pango_font_description_set_size (desc, size * PANGO_SCALE);
+	  pango_layout_set_font_description (layout, desc);
+	  pango_font_description_free (desc);
+	}
+
+      pango_layout_get_pixel_extents (layout, NULL, &logical_rect);
 
       if (render_cb)
 	(*render_cb) (layout, x, y+*height, cb_context, cb_data);
 
-      *width = MAX (*width, PANGO_PIXELS (logical_rect.x + logical_rect.width));
-      *width = MAX (*width, PANGO_PIXELS (pango_layout_get_width (layout)));
-      *height += PANGO_PIXELS (logical_rect.height);
-
-      g_object_unref (layout);
+      *width = MAX (*width, 
+		    MAX (logical_rect.x + logical_rect.width,
+			 PANGO_PIXELS (pango_layout_get_width (layout))));
+      *height +=    MAX (logical_rect.y + logical_rect.height,
+			 PANGO_PIXELS (pango_layout_get_height (layout)));
     }
 }
 
@@ -258,7 +262,9 @@ do_output (PangoContext     *context,
 
   set_transform (context, transform_cb, cb_context, cb_data, NULL);
 
-  pango_context_set_language (context, pango_language_get_default ());
+  pango_context_set_language (context,
+			      opt_language ? pango_language_from_string (opt_language)
+					   : pango_language_get_default ());
   pango_context_set_base_dir (context,
 			      opt_rtl ? PANGO_DIRECTION_RTL : PANGO_DIRECTION_LTR);
 
@@ -292,10 +298,11 @@ do_output (PangoContext     *context,
   pango_context_set_base_gravity (context, opt_gravity);
   pango_context_set_gravity_hint (context, opt_gravity_hint);
 
+  layout = make_layout (context, text, -1);
+
   set_transform (context, transform_cb, cb_context, cb_data, &matrix);
 
-  output_body (context,
-	       text,
+  output_body (layout,
 	       NULL, NULL, NULL,
 	       &rotated_width, &rotated_height,
 	       supports_matrix);
@@ -312,8 +319,7 @@ do_output (PangoContext     *context,
   set_transform (context, transform_cb, cb_context, cb_data, &matrix);
 
   if (render_cb)
-    output_body (context,
-		 text,
+    output_body (layout,
 		 render_cb, cb_context, cb_data,
 		 &rotated_width, &rotated_height,
 		 supports_matrix);
@@ -331,6 +337,7 @@ do_output (PangoContext     *context,
 
   pango_context_set_matrix (context, orig_matrix);
   pango_matrix_free (orig_matrix);
+  g_object_unref (layout);
 }
 
 static gboolean
@@ -338,7 +345,7 @@ parse_enum (GType       type,
 	    int        *value,
 	    const char *name,
 	    const char *arg,
-	    gpointer    data,
+	    gpointer    data G_GNUC_UNUSED,
 	    GError **error)
 {
   char *possible_values = NULL;
@@ -366,13 +373,23 @@ parse_enum (GType       type,
   return ret;
 }
 
+static gboolean
+parse_align (const char *name,
+	     const char *arg,
+	     gpointer    data,
+	     GError **error)
+{
+  return parse_enum (PANGO_TYPE_ALIGNMENT, (int*)(void*)&opt_align,
+		     name, arg, data, error);
+}
+
 static gboolean
 parse_ellipsis (const char *name,
 		const char *arg,
 		gpointer    data,
 		GError **error)
 {
-  return parse_enum (PANGO_TYPE_ELLIPSIZE_MODE, &opt_ellipsize,
+  return parse_enum (PANGO_TYPE_ELLIPSIZE_MODE, (int*)(void*)&opt_ellipsize,
 		     name, arg, data, error);
 }
 
@@ -382,7 +399,7 @@ parse_gravity (const char *name,
 	       gpointer    data,
 	       GError **error)
 {
-  return parse_enum (PANGO_TYPE_GRAVITY, &opt_gravity,
+  return parse_enum (PANGO_TYPE_GRAVITY, (int*)(void*)&opt_gravity,
 		     name, arg, data, error);
 }
 
@@ -392,14 +409,14 @@ parse_gravity_hint (const char *name,
 		    gpointer    data,
 		    GError **error)
 {
-  return parse_enum (PANGO_TYPE_GRAVITY_HINT, &opt_gravity_hint,
+  return parse_enum (PANGO_TYPE_GRAVITY_HINT, (int*)(void*)&opt_gravity_hint,
 		     name, arg, data, error);
 }
 
 static gboolean
-parse_hinting (const char *name,
+parse_hinting (const char *name G_GNUC_UNUSED,
 	       const char *arg,
-	       gpointer    data,
+	       gpointer    data G_GNUC_UNUSED,
 	       GError    **error)
 {
   gboolean ret = TRUE;
@@ -429,7 +446,7 @@ parse_wrap (const char *name,
 	    GError    **error)
 {
   gboolean ret;
-  if ((ret = parse_enum (PANGO_TYPE_WRAP_MODE, &opt_wrap,
+  if ((ret = parse_enum (PANGO_TYPE_WRAP_MODE, (int*)(void*)&opt_wrap,
 			 name, arg, data, error)))
     {
       opt_wrap_set = TRUE;
@@ -485,11 +502,10 @@ backend_description (void)
 
 }
 
-
 static gboolean
-parse_backend (const char *name,
+parse_backend (const char *name G_GNUC_UNUSED,
 	       const char *arg,
-	       gpointer    data,
+	       gpointer    data G_GNUC_UNUSED,
 	       GError    **error)
 {
   gboolean ret = TRUE;
@@ -519,10 +535,10 @@ parse_backend (const char *name,
 
 
 static gboolean
-show_version(const char *name,
-	     const char *arg,
-	     gpointer    data,
-	     GError    **error)
+show_version(const char *name G_GNUC_UNUSED,
+	     const char *arg G_GNUC_UNUSED,
+	     gpointer    data G_GNUC_UNUSED,
+	     GError    **error G_GNUC_UNUSED)
 {
   g_printf("%s (%s) %s\n", prog_name, PACKAGE_NAME, PACKAGE_VERSION);
   g_printf("module interface version: %s\n", MODULE_VERSION);
@@ -536,7 +552,7 @@ show_version(const char *name,
 void
 parse_options (int argc, char *argv[])
 {
-  gchar *backend_options = backends_to_string();
+  gchar *backend_options = backends_to_string ();
   GOptionFlags backend_flag = backends_get_count () > 1 ? 0 : G_OPTION_FLAG_HIDDEN;
   gchar *backend_desc = backend_description ();
   GOptionEntry entries[] =
@@ -549,6 +565,8 @@ parse_options (int argc, char *argv[])
      "Do not display (just write to file or whatever)",			NULL},
     {"dpi",		0, 0, G_OPTION_ARG_INT,				&opt_dpi,
      "Set the resolution",					    "number"},
+    {"align",		0, 0, G_OPTION_ARG_CALLBACK,			&parse_align,
+     "Text alignment",				         "left/center/right"},
     {"ellipsize",	0, 0, G_OPTION_ARG_CALLBACK,			&parse_ellipsis,
      "Ellipsization mode",				  "start/middle/end"},
     {"font",		0, 0, G_OPTION_ARG_STRING,			&opt_font,
@@ -559,10 +577,16 @@ parse_options (int argc, char *argv[])
      "Gravity hint",				       "natural/strong/line"},
     {"header",		0, 0, G_OPTION_ARG_NONE,			&opt_header,
      "Display the options in the output",				NULL},
+    {"height",		0, 0, G_OPTION_ARG_INT,				&opt_height,
+     "Height in points (positive) or number of lines (negative) for ellipsizing", "+points/-numlines"},
     {"hinting",		0, 0, G_OPTION_ARG_CALLBACK,			&parse_hinting,
      "Hinting style",					    "none/auto/full"},
     {"indent",		0, 0, G_OPTION_ARG_INT,				&opt_indent,
      "Width in points to indent paragraphs",			    "points"},
+    {"justify",		0, 0, G_OPTION_ARG_NONE,			&opt_justify,
+     "Align paragraph lines to be justified",			    	NULL},
+    {"language",	0, 0, G_OPTION_ARG_STRING,			&opt_language,
+     "Language to use for font selection",			    "en_US/etc"},
     {"margin",		0, 0, G_OPTION_ARG_INT,				&opt_margin,
      "Set the margin on the output in pixels",			    "pixels"},
     {"markup",		0, 0, G_OPTION_ARG_NONE,			&opt_markup,
@@ -577,6 +601,8 @@ parse_options (int argc, char *argv[])
      "Angle at which to rotate results",			   "degrees"},
     {"runs",		'n', 0, G_OPTION_ARG_INT,			&opt_runs,
      "Run Pango layout engine this many times",			   "integer"},
+    {"single-par",	0, 0, G_OPTION_ARG_NONE,			&opt_single_par,
+     "Enable single-paragraph mode",					NULL},
     {"text",		't', 0, G_OPTION_ARG_STRING,			&opt_text,
      "Text to display (instead of a file)",			    "string"},
     {"version",		0, G_OPTION_FLAG_NO_ARG, G_OPTION_ARG_CALLBACK, &show_version,
@@ -584,7 +610,7 @@ parse_options (int argc, char *argv[])
     {"waterfall",	0, 0, G_OPTION_ARG_NONE,			&opt_waterfall,
      "Create a waterfall display",					NULL},
     {"width",		'w', 0, G_OPTION_ARG_INT,			&opt_width,
-     "Width in points to which to wrap output",			    "points"},
+     "Width in points to which to wrap lines or ellipsize",	    "points"},
     {"wrap",		0, 0, G_OPTION_ARG_CALLBACK,			&parse_wrap,
      "Text wrapping mode (needs a width to be set)",   "word/char/word-char"},
     {NULL}
@@ -626,12 +652,6 @@ parse_options (int argc, char *argv[])
 	fail ("No viewer backend found");
     }
 
-  /* if wrap mode is set then width must be set */
-  if (opt_width < 0 && opt_wrap_set)
-    {
-      g_printerr ("The wrap mode only has effect if a width is set\n");
-    }
-
   /* Get the text
    */
   if (opt_text)
