@@ -182,8 +182,8 @@ LANGUAGE(
 	)
 LANGUAGE(
 	 fa	/* Persian */,
-	 MISC	/* Behdad Esfahbod */,
-	 "«اَلا یا اَیُّها السّاقی اَدِر کَأساً وَ ناوِلْهٰا»، که عشق آسان نمود اول ولی افتاد مشکل‌ها."
+	 MISC	/* Behdad Esfahbod (#548730) */,
+	 "«الا یا اَیُّها السّاقی! اَدِرْ کَأساً وَ ناوِلْهٰا!» که عشق آسان نمود اوّل، ولی افتاد مشکل‌ها!"
 	)
 LANGUAGE(
 	 fi	/* Finnish */,
@@ -247,9 +247,9 @@ LANGUAGE(
 	)
 LANGUAGE(
 	 hi	/* Hindi */,
-	 GLASS,
-	 "मैं काँच खा सकता हूँ, मुझे उस से कोई पीडा नहीं होती."
-	 /* I can eat glass and it doesn't hurt me. */
+	 MISC	/* G Karunakar (#549532) */,
+	 "नहीं नजर किसी की बुरी नहीं किसी का मुँह काला जो करे सो उपर वाला"
+	 /* its not in the sight or the face, but its all in god's grace. */
 	)
 LANGUAGE(
 	 hr	/* Croatian */,
@@ -284,7 +284,7 @@ LANGUAGE(
 LANGUAGE(
 	 ja	/* Japanese */,
 	 KERMIT,
-	 "いろはにほへど　ちりぬるを　色は匂へど 散りぬるを"
+	 "いろはにほへと ちりぬるを 色は匂へど 散りぬるを"
 	)
 LANGUAGE(
 	 jam	/* Jamaican Creole English */,
@@ -434,9 +434,9 @@ LANGUAGE(
 	)
 LANGUAGE(
 	 ro	/* Romanian */,
-	 WP-SFD,
-	 "Agera vulpe maronie sare peste câinele cel leneş."
-	 /* The quick brown fox jumps over the lazy dog. */
+	 MISC	/* Misu Moldovan (#552993) */,
+	 "Fumegând hipnotic sașiul azvârle mreje în bălți."
+	 /* Hypnotically smoking, the cross-eyed man throws fishing nets into ponds. */
 	)
 LANGUAGE(
 	 ru	/* Russian */,
